@@ -1,10 +1,11 @@
+// -*- C++ -*-
 // Module:  Log4CPLUS
 // File:    sleep.h
 // Created: 5/2003
 // Author:  Tad E. Smith
 //
 //
-// Copyright 2003-2009 Tad E. Smith
+// Copyright 2003-2015 Tad E. Smith
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -20,11 +21,15 @@
 
 /** @file */
 
-#ifndef _LOG4CPLUS_HELPERS_SLEEP_HEADER_
-#define _LOG4CPLUS_HELPERS_SLEEP_HEADER_
+#ifndef LOG4CPLUS_HELPERS_SLEEP_HEADER_
+#define LOG4CPLUS_HELPERS_SLEEP_HEADER_
 
 #include <log4cplus/config.hxx>
 
+#if defined (LOG4CPLUS_HAVE_PRAGMA_ONCE)
+#pragma once
+#endif
+
 
 namespace log4cplus {
     namespace helpers {
@@ -34,5 +39,5 @@ namespace log4cplus {
     } // end namespace helpers
 } // end namespace log4cplus
 
-#endif // _LOG4CPLUS_HELPERS_SLEEP_HEADER_
+#endif // LOG4CPLUS_HELPERS_SLEEP_HEADER_
 
