@@ -1,10 +1,11 @@
+// -*- C++ -*-
 // Module:  Log4CPLUS
 // File:    Layout.h
 // Created: 6/2001
 // Author:  Tad E. Smith
 //
 //
-// Copyright 2001-2009 Tad E. Smith
+// Copyright 2001-2015 Tad E. Smith
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -20,17 +21,18 @@
 
 /** @file */
 
-#ifndef _LOG4CPLUS_LAYOUT_HEADER_
-#define _LOG4CPLUS_LAYOUT_HEADER_
+#ifndef LOG4CPLUS_LAYOUT_HEADER_
+#define LOG4CPLUS_LAYOUT_HEADER_
 
 #include <log4cplus/config.hxx>
+
+#if defined (LOG4CPLUS_HAVE_PRAGMA_ONCE)
+#pragma once
+#endif
+
 #include <log4cplus/loglevel.h>
 #include <log4cplus/streams.h>
 #include <log4cplus/tstring.h>
-#include <log4cplus/helpers/logloguser.h>
-#include <log4cplus/helpers/property.h>
-#include <log4cplus/helpers/timehelper.h>
-#include <log4cplus/spi/loggingevent.h>
 
 #include <vector>
 
@@ -38,8 +40,28 @@
 namespace log4cplus {
 
     // Forward Declarations
-    namespace pattern {
+    namespace pattern
+    {
+
         class PatternConverter;
+
+    }
+
+
+    namespace helpers
+    {
+
+        class Properties;
+        class Time;
+
+    }
+
+
+    namespace spi
+    {
+
+        class InternalLoggingEvent;
+
     }
 
 
@@ -47,22 +69,23 @@ namespace log4cplus {
      * This class is used to layout strings sent to an {@link
      * log4cplus::Appender}.
      */
-    class LOG4CPLUS_EXPORT Layout : protected :: log4cplus::helpers::LogLogUser {
+    class LOG4CPLUS_EXPORT Layout
+    {
     public:
-        Layout() : llmCache(getLogLevelManager()) {}
-        Layout(const log4cplus::helpers::Properties&) 
-          : llmCache(getLogLevelManager())  {}
-        virtual ~Layout() {}
+        Layout();
+        Layout(const helpers::Properties& properties);
+        virtual ~Layout() = 0;
+
+        virtual void formatAndAppend(log4cplus::tostream& output,
+            const log4cplus::spi::InternalLoggingEvent& event) = 0;
 
-        virtual void formatAndAppend(log4cplus::tostream& output, 
-                                     const log4cplus::spi::InternalLoggingEvent& event) = 0;
     protected:
         LogLevelManager& llmCache;
-        
+
     private:
       // Disable copy
         Layout(const Layout&);
-        Layout& operator=(Layout&);
+        Layout& operator=(Layout const &);
     };
 
 
@@ -77,15 +100,18 @@ namespace log4cplus {
      *
      * {@link PatternLayout} offers a much more powerful alternative.
      */
-    class LOG4CPLUS_EXPORT SimpleLayout : public Layout {
+    class LOG4CPLUS_EXPORT SimpleLayout
+        : public Layout
+    {
     public:
-        SimpleLayout() {}
-        SimpleLayout(const log4cplus::helpers::Properties& properties) : Layout(properties) {}
+        SimpleLayout();
+        SimpleLayout(const log4cplus::helpers::Properties& properties);
+        virtual ~SimpleLayout();
 
-        virtual void formatAndAppend(log4cplus::tostream& output, 
+        virtual void formatAndAppend(log4cplus::tostream& output,
                                      const log4cplus::spi::InternalLoggingEvent& event);
 
-    private: 
+    private:
       // Disallow copying of instances of this class
         SimpleLayout(const SimpleLayout&);
         SimpleLayout& operator=(const SimpleLayout&);
@@ -96,108 +122,131 @@ namespace log4cplus {
     /**
      * TTCC layout format consists of time, thread, Logger and nested
      * diagnostic context information, hence the name.
-     * 
+     *
      * The time format depends on the <code>DateFormat</code> used.  Use the
-     * <code>Use_gmtime</code> to specify whether messages should be logged using
-     * <code>localtime</code> or <code>gmtime</code>.
-     * 
+     * <code>Use_gmtime</code> to specify whether messages should be logged
+     * using <code>localtime</code> or <code>gmtime</code>. There are also
+     * <code>ThreadPrinting</code>, <code>CategoryPrefixing</code> and
+     * <code>ContextPrinting</code> properties to turn on and off thread name,
+     * logger name and NDC context printing respectively.
+     *
      * Here is an example TTCCLayout output:
-     * 
-     * <pre>
-     * 176 [main] INFO  org.apache.log4j.examples.Sort - Populating an array of 2 elements in reverse order.
-     * 225 [main] INFO  org.apache.log4j.examples.SortAlgo - Entered the sort method.
-     * 262 [main] DEBUG org.apache.log4j.examples.SortAlgo.OUTER i=1 - Outer loop.
-     * 276 [main] DEBUG org.apache.log4j.examples.SortAlgo.SWAP i=1 j=0 - Swapping intArray[0] = 1 and intArray[1] = 0
-     * 290 [main] DEBUG org.apache.log4j.examples.SortAlgo.OUTER i=0 - Outer loop.
-     * 304 [main] INFO  org.apache.log4j.examples.SortAlgo.DUMP - Dump of interger array:
-     * 317 [main] INFO  org.apache.log4j.examples.SortAlgo.DUMP - Element [0] = 0
-     * 331 [main] INFO  org.apache.log4j.examples.SortAlgo.DUMP - Element [1] = 1
-     * 343 [main] INFO  org.apache.log4j.examples.Sort - The next log statement should be an error message.
-     * 346 [main] ERROR org.apache.log4j.examples.SortAlgo.DUMP - Tried to dump an uninitialized array.
-     * 467 [main] INFO  org.apache.log4j.examples.Sort - Exiting main method.
-     * </pre>
-     * 
-     *  The first field is the number of milliseconds elapsed since the
-     *  start of the program. The second field is the thread outputting the
-     *  log statement. The third field is the LogLevel, the fourth field is
-     *  the logger to which the statement belongs.
-     * 
-     *  The fifth field (just before the '-') is the nested diagnostic
-     *  context.  Note the nested diagnostic context may be empty as in the
-     *  first two statements. The text after the '-' is the message of the
-     *  statement.
-     * 
+     *
+     * ~~~~
+     * 1 [0x60004dca0] WARN test.TestThread <> - Thread-3 TestThread.run()- Starting...
+     * 1 [0x60004dca0] TRACE SlowObject <Thread-3 loop> - ENTER: SlowObject::doSomething()
+     * 2 [0x60004b030] INFO SlowObject <Thread-0 loop> - Actually doing something...1, 2, 3, testing...DONE
+     * 2 [0x60004b130] INFO SlowObject <Thread-1 loop> - Actually doing something...
+     * 2 [0x60004b030] TRACE SlowObject <Thread-0 loop> - EXIT:  SlowObject::doSomething()
+     * 2 [0x60004b030] TRACE SlowObject <Thread-0 loop> - ENTER: SlowObject::doSomething()
+     * 3 [0x60004b130] INFO SlowObject <Thread-1 loop> - Actually doing something...1, 2, 3, testing...DONE
+     * 3 [0x60004cad0] INFO SlowObject <Thread-2 loop> - Actually doing something...
+     * ~~~~
+     *
+     *  The first field is the number of milliseconds elapsed since
+     *  the start of the program.
+     *
+     *  The second field is the thread outputting the log
+     *  statement. (The value is the same as that of the `t` formatter
+     *  for PatternLayout.)
+     *
+     *  The third field is the LogLevel.
+     *
+     *  The fourth field is the logger to which the statement belongs.
+     *
+     *  The fifth field (just before the '-') is the nested
+     *  diagnostic context.  Note the nested diagnostic context may be
+     *  empty as in the first two statements. The text after the '-'
+     *  is the message of the statement.
+     *
      *  PatternLayout offers a much more flexible alternative.
      */
-    class LOG4CPLUS_EXPORT TTCCLayout : public Layout {
+    class LOG4CPLUS_EXPORT TTCCLayout
+        : public Layout
+    {
     public:
-      // Ctor and dtor
-        TTCCLayout(bool use_gmtime = false);
+        TTCCLayout(bool use_gmtime = false, bool thread_printing = true,
+            bool category_prefixes = true, bool context_printing = true);
         TTCCLayout(const log4cplus::helpers::Properties& properties);
         virtual ~TTCCLayout();
 
-        virtual void formatAndAppend(log4cplus::tostream& output, 
+        virtual void formatAndAppend(log4cplus::tostream& output,
                                      const log4cplus::spi::InternalLoggingEvent& event);
 
+        bool getThreadPrinting() const;
+        void setThreadPrinting(bool);
+
+        bool getCategoryPrefixing() const;
+        void setCategoryPrefixing(bool);
+
+        bool getContextPrinting() const;
+        void setContextPrinting(bool);
+
     protected:
        log4cplus::tstring dateFormat;
        bool use_gmtime;
-     
-    private: 
+       bool thread_printing;
+       bool category_prefixing;
+       bool context_printing;
+
+    private:
       // Disallow copying of instances of this class
         TTCCLayout(const TTCCLayout&);
         TTCCLayout& operator=(const TTCCLayout&);
     };
 
 
+    LOG4CPLUS_EXPORT helpers::Time const & getTTCCLayoutTimeBase ();
 
 
     /**
      * A flexible layout configurable with pattern string.
-     * 
-     * The goal of this class is to format a InternalLoggingEvent and return 
-     * the results as a string. The results depend on the <em>conversion 
+     *
+     * The goal of this class is to format a InternalLoggingEvent and return
+     * the results as a string. The results depend on the <em>conversion
      * pattern</em>.
-     * 
+     *
      * The conversion pattern is closely related to the conversion
      * pattern of the printf function in C. A conversion pattern is
      * composed of literal text and format control expressions called
      * <em>conversion specifiers</em>.
-     *  
+     *
      * <i>You are free to insert any literal text within the conversion
      * pattern.</i>
-     * 
+     *
      * Each conversion specifier starts with a percent sign (%%) and is
      * followed by optional <em>format modifiers</em> and a <em>conversion
      * character</em>. The conversion character specifies the type of
      * data, e.g. Logger, LogLevel, date, thread name. The format
      * modifiers control such things as field width, padding, left and
      * right justification. The following is a simple example.
-     * 
-     * Let the conversion pattern be <b>"%-5p [%t]: %m%n"</b> and assume
+     *
+     * Let the conversion pattern be `"%-5p [%t]: %m%n"` and assume
      * that the log4cplus environment was set to use a PatternLayout. Then the
      * statements
-     * <code><pre>
+     *
+     * ~~~~{.c}
      * Logger root = Logger.getRoot();
      * LOG4CPLUS_DEBUG(root, "Message 1");
      * LOG4CPLUS_WARN(root, "Message 2");
-     * </pre></code>
+     * ~~~~
+     *
      * would yield the output
-     * <tt><pre>
+     *
+     * ~~~~
      * DEBUG [main]: Message 1
-     * WARN  [main]: Message 2  
-     * </pre></tt>
-     * 
+     * WARN  [main]: Message 2
+     * ~~~~
+     *
      * Note that there is no explicit separator between text and
      * conversion specifiers. The pattern parser knows when it has reached
      * the end of a conversion specifier when it reads a conversion
      * character. In the example above the conversion specifier
      * <b>"%-5p"</b> means the LogLevel of the logging event should be left
      * justified to a width of five characters.
-     * 
+     *
      * The recognized conversion characters are
-     * 
-     * 
+     *
      * <table border="1" CELLPADDING="8">
      * <tr>
      * <td>Conversion Character</td>
@@ -213,30 +262,30 @@ namespace log4cplus {
      *
      * <tr>
      *   <td align=center><b>c</b></td>
-     * 
+     *
      *   <td>Used to output the logger of the logging event. The
      *   logger conversion specifier can be optionally followed by
      *   <em>precision specifier</em>, that is a decimal constant in
      *   brackets.
-     * 
+     *
      *   If a precision specifier is given, then only the corresponding
      *   number of right most components of the logger name will be
      *   printed. By default the logger name is printed in full.
-     * 
+     *
      *   For example, for the logger name "a.b.c" the pattern
      *   <b>%c{2}</b> will output "b.c".
-     * 
+     *
      * </td>
      * </tr>
-     * 
-     * <tr> 
-     *   <td align=center><b>d</b></td> 
      *
-     *   <td>Used to output the date of the logging event in <b>UTC</b>. 
+     * <tr>
+     *   <td align=center><b>d</b></td>
+     *
+     *   <td>Used to output the date of the logging event in <b>UTC</b>.
      *
      *   The date conversion specifier may be followed by a <em>date format
      *   specifier</em> enclosed between braces. For example, <b>%%d{%%H:%%M:%%s}</b>
-     *   or <b>%%d{%%d&nbsp;%%b&nbsp;%%Y&nbsp;%%H:%%M:%%s}</b>.  If no date format 
+     *   or <b>%%d{%%d&nbsp;%%b&nbsp;%%Y&nbsp;%%H:%%M:%%s}</b>.  If no date format
      *   specifier is given then <b>%%d{%%d&nbsp;%%m&nbsp;%%Y&nbsp;%%H:%%M:%%s}</b>
      *   is assumed.
      *
@@ -272,29 +321,40 @@ namespace log4cplus {
      *   found in the <code>&lt;ctime&gt;</code> header for more information.
      * </td>
      * </tr>
-     * 
-     * <tr> 
-     *   <td align=center><b>D</b></td> 
+     *
+     * <tr>
+     *   <td align=center><b>D</b></td>
      *
      *   <td>Used to output the date of the logging event in <b>local</b> time.
      *
      *   All of the above information applies.
      * </td>
      * </tr>
-     * 
+     *
+     * <tr>
+     *   <td align=center><b>E</b></td>
+     *
+     *   <td>Used to output the value of a given environment variable.  The
+     *   name of is supplied as an argument in brackets.  If the variable does
+     *   exist then empty string will be used.
+     *
+     *   For example, the pattern <b>%E{HOME}</b> will output the contents
+     *   of the HOME environment variable.
+     * </td>
+     * </tr>
+     *
      * <tr>
      *   <td align=center><b>F</b></td>
-     * 
+     *
      *   <td>Used to output the file name where the logging request was
      *   issued.
-     * 
+     *
      *   <b>NOTE</b> Unlike log4j, there is no performance penalty for
-     *   calling this method.
-     * 
+     *   calling this method.</td>
      * </tr>
-     * 
-     * <tr> 
-     *   <td align=center><b>h</b></td> 
+     *
+     * <tr>
+     *   <td align=center><b>h</b></td>
      *
      *   <td>Used to output the hostname of this system (as returned
      *   by gethostname(2)).
@@ -304,9 +364,9 @@ namespace log4cplus {
      *
      * </td>
      * </tr>
-     * 
-     * <tr> 
-     *   <td align=center><b>H</b></td> 
+     *
+     * <tr>
+     *   <td align=center><b>H</b></td>
      *
      *   <td>Used to output the fully-qualified domain name of this
      *   system (as returned by gethostbyname(2) for the hostname
@@ -317,52 +377,80 @@ namespace log4cplus {
      *
      * </td>
      * </tr>
-     * 
+     *
      * <tr>
      * <td align=center><b>l</b></td>
-     * 
+     *
      *   <td>Equivalent to using "%F:%L"
-     * 
+     *
      *   <b>NOTE:</b> Unlike log4j, there is no performance penalty for
      *   calling this method.
-     * 
+     *
      * </td>
      * </tr>
-     * 
+     *
      * <tr>
      *   <td align=center><b>L</b></td>
-     * 
+     *
      *   <td>Used to output the line number from where the logging request
      *   was issued.
-     * 
+     *
      *   <b>NOTE:</b> Unlike log4j, there is no performance penalty for
      *   calling this method.
-     * 
+     *
      * </tr>
-     * 
-     *  
+     *
      * <tr>
      *   <td align=center><b>m</b></td>
-     *   <td>Used to output the application supplied message associated with 
-     *   the logging event.</td>   
+     *   <td>Used to output the application supplied message associated with
+     *   the logging event.</td>
      * </tr>
-     * 
+     *
+     * <tr>
+     *   <td align=center><b>M</b></td>
+     *
+     *   <td>Used to output function name using
+     *   <code>__FUNCTION__</code> or similar macro.
+     *
+     *   <b>NOTE</b> The <code>__FUNCTION__</code> macro is not
+     *   standard but it is common extension provided by all compilers
+     *   (as of 2010). In case it is missing or in case this feature
+     *   is disabled using the
+     *   <code>LOG4CPLUS_DISABLE_FUNCTION_MACRO</code> macro, %M
+     *   expands to an empty string.</td>
+     * </tr>
+     *
      * <tr>
      *   <td align=center><b>n</b></td>
-     *   
+     *
      *   <td>Outputs the platform dependent line separator character or
-     *   characters. 
+     *   characters.
      * </tr>
-     * 
+     *
      * <tr>
      *   <td align=center><b>p</b></td>
      *   <td>Used to output the LogLevel of the logging event.</td>
      * </tr>
-     * 
+     *
+     * <tr>
+     *   <td align=center><b>r</b></td>
+     *   <td>Used to output miliseconds since program start
+     *   of the logging event.</td>
+     * </tr>
+     *
      * <tr>
      *   <td align=center><b>t</b></td>
-     * 
-     *   <td>Used to output the name of the thread that generated the
+     *
+     *   <td>Used to output the thread ID of the thread that generated
+     *   the logging event. (This is either `pthread_t` value returned
+     *   by `pthread_self()` on POSIX platforms or thread ID returned
+     *   by `GetCurrentThreadId()` on Windows.)</td>
+     * </tr>
+     *
+     * <tr>
+     *   <td align=center><b>T</b></td>
+     *
+     *   <td>Used to output alternative name of the thread that generated the
      *   logging event.</td>
      * </tr>
      *
@@ -375,27 +463,38 @@ namespace log4cplus {
      *
      * <tr>
      *   <td align=center><b>x</b></td>
-     * 
+     *
      *   <td>Used to output the NDC (nested diagnostic context) associated
      *   with the thread that generated the logging event.
-     *   </td>     
+     *   </td>
+     * </tr>
+     *
+     * <tr>
+     *   <td align=center><b>X</b></td>
+     *
+     *   <td>Used to output the MDC (mapped diagnostic context)
+     *   associated with the thread that generated the logging
+     *   event. It takes optional key parameter. Without the key
+     *   paramter (%%X), it outputs the whole MDC map. With the key
+     *   (%%X{key}), it outputs just the key's value.
+     *   </td>
      * </tr>
      *
      * <tr>
      *   <td align=center><b>"%%"</b></td>
      *   <td>The sequence "%%" outputs a single percent sign.
-     *   </td>     
+     *   </td>
      * </tr>
-     *  
+     *
      * </table>
-     * 
+     *
      * By default the relevant information is output as is. However,
      * with the aid of format modifiers it is possible to change the
      * minimum field width, the maximum field width and justification.
-     * 
+     *
      * The optional format modifier is placed between the percent sign
      * and the conversion character.
-     * 
+     *
      * The first optional format modifier is the <em>left justification
      * flag</em> which is just the minus (-) character. Then comes the
      * optional <em>minimum field width</em> modifier. This is a decimal
@@ -407,7 +506,7 @@ namespace log4cplus {
      * padding character is space. If the data item is larger than the
      * minimum field width, the field is expanded to accommodate the
      * data. The value is never truncated.
-     * 
+     *
      * This behavior can be changed using the <em>maximum field
      * width</em> modifier which is designated by a period followed by a
      * decimal constant. If the data item is longer than the maximum
@@ -417,11 +516,11 @@ namespace log4cplus {
      * ten characters long, then the first two characters of the data item
      * are dropped. This behavior deviates from the printf function in C
      * where truncation is done from the end.
-     * 
+     *
      * Below are various format modifier examples for the logger
      * conversion specifier.
-     * 
-     * 
+     *
+     *
      * <TABLE BORDER=1 CELLPADDING=8>
      * <tr>
      * <td>Format modifier</td>
@@ -436,79 +535,93 @@ namespace log4cplus {
      * <td align=center>false</td>
      * <td align=center>20</td>
      * <td align=center>none</td>
-     * 
+     *
      * <td>Left pad with spaces if the logger name is less than 20
      * characters long.
      * </tr>
-     * 
+     *
      * <tr> <td align=center>%-20c</td> <td align=center>true</td> <td
      * align=center>20</td> <td align=center>none</td> <td>Right pad with
      * spaces if the logger name is less than 20 characters long.
      * </tr>
-     * 
+     *
      * <tr>
      * <td align=center>%.30c</td>
      * <td align=center>NA</td>
      * <td align=center>none</td>
      * <td align=center>30</td>
-     * 
+     *
      * <td>Truncate from the beginning if the logger name is longer than 30
      * characters.
      * </tr>
-     * 
+     *
      * <tr>
      * <td align=center>%20.30c</td>
      * <td align=center>false</td>
      * <td align=center>20</td>
      * <td align=center>30</td>
-     * 
+     *
      * <td>Left pad with spaces if the logger name is shorter than 20
      * characters. However, if logger name is longer than 30 characters,
-     * then truncate from the beginning.   
+     * then truncate from the beginning.
      * </tr>
-     * 
+     *
      * <tr>
      * <td align=center>%-20.30c</td>
      * <td align=center>true</td>
      * <td align=center>20</td>
      * <td align=center>30</td>
-     *  
+     *
      * <td>Right pad with spaces if the logger name is shorter than 20
      * characters. However, if logger name is longer than 30 characters,
-     * then truncate from the beginning.   
+     * then truncate from the beginning.
      * </tr>
-     * 
+     *
      * </table>
-     * 
+     *
      * Below are some examples of conversion patterns.
-     * 
+     *
      * <dl>
-     * 
-     * <dt><b>"%r [%t] %-5p %c %x - %m%n"</b> 
+     *
+     * <dt><b>"%r [%t] %-5p %c %x - %m%n"</b>
      * <dd>This is essentially the TTCC layout.
-     * 
+     *
      * <dt><b>"%-6r [%15.15t] %-5p %30.30c %x - %m%n"</b>
-     * 
+     *
      * <dd>Similar to the TTCC layout except that the relative time is
      * right padded if less than 6 digits, thread name is right padded if
      * less than 15 characters and truncated if longer and the logger
      * name is left padded if shorter than 30 characters and truncated if
      * longer.
-     *  
+     *
      * </dl>
-     * 
+     *
      * The above text is largely inspired from Peter A. Darnell and
      * Philip E. Margolis' highly recommended book "C -- a Software
      * Engineering Approach", ISBN 0-387-97389-3.
+     *
+     * <h3>Properties</h3>
+     *
+     * <dl>
+     * <dt><tt>NDCMaxDepth</tt></dt>
+     * <dd>This property limits how many deepest NDC components will
+     * be printed by <b>%%x</b> specifier.</dd>
+     *
+     * <dt><tt>ConversionPattern</tt></dt>
+     * <dd>This property specifies conversion pattern.</dd>
+     * </dl>
+     *
      */
-    class LOG4CPLUS_EXPORT PatternLayout : public Layout {
+    class LOG4CPLUS_EXPORT PatternLayout
+        : public Layout
+    {
     public:
       // Ctors and dtor
         PatternLayout(const log4cplus::tstring& pattern);
         PatternLayout(const log4cplus::helpers::Properties& properties);
         virtual ~PatternLayout();
 
-        virtual void formatAndAppend(log4cplus::tostream& output, 
+        virtual void formatAndAppend(log4cplus::tostream& output,
                                      const log4cplus::spi::InternalLoggingEvent& event);
 
     protected:
@@ -518,7 +631,7 @@ namespace log4cplus {
         log4cplus::tstring pattern;
         std::vector<pattern::PatternConverter*> parsedPattern;
 
-    private: 
+    private:
       // Disallow copying of instances of this class
         PatternLayout(const PatternLayout&);
         PatternLayout& operator=(const PatternLayout&);
@@ -528,5 +641,4 @@ namespace log4cplus {
 
 } // end namespace log4cplus
 
-#endif // _LOG4CPLUS_LAYOUT_HEADER_
-
+#endif // LOG4CPLUS_LAYOUT_HEADER_
